@@ -6,7 +6,6 @@
 #include "core/except.h"
 
 #ifdef ALSOFT_EAX
-#include "al/eax/call.h"
 #include "al/eax/effect.h"
 #endif // ALSOFT_EAX
 
@@ -23,6 +22,7 @@ public:
     [[gnu::format(printf, 3, 4)]]
 #endif
     effect_exception(ALenum code, const char *msg, ...);
+    ~effect_exception() override;
 
     ALenum errorCode() const noexcept { return mErrorCode; }
 };
@@ -87,7 +87,7 @@ extern const EffectVtable ConvolutionEffectVtable;
 
 
 #ifdef ALSOFT_EAX
-EaxEffectUPtr eax_create_eax_effect(ALenum al_effect_type, const EaxCall& call);
+EaxEffectUPtr eax_create_eax_effect(ALenum al_effect_type, int eax_version);
 #endif // ALSOFT_EAX
 
 #endif /* AL_EFFECTS_EFFECTS_H */
