@@ -1,50 +1,65 @@
 
 #include "config.h"
 
+#include <algorithm>
+#include <array>
+#include <complex>
+#include <cstddef>
+#include <functional>
+#include <iterator>
+#include <memory>
+#include <stdint.h>
+#include <utility>
+
 #ifdef HAVE_SSE_INTRINSICS
 #include <xmmintrin.h>
+#elif defined(HAVE_NEON)
+#include <arm_neon.h>
 #endif
 
-#include "AL/al.h"
-#include "AL/alc.h"
-
-#include "al/auxeffectslot.h"
-#include "alcmain.h"
+#include "albyte.h"
 #include "alcomplex.h"
-#include "alcontext.h"
 #include "almalloc.h"
+#include "alnumbers.h"
+#include "alnumeric.h"
 #include "alspan.h"
-#include "ambidefs.h"
-#include "bformatdec.h"
-#include "buffer_storage.h"
-#include "effects/base.h"
-#include "filters/splitter.h"
-#include "fmt_traits.h"
-#include "logging.h"
+#include "base.h"
+#include "core/ambidefs.h"
+#include "core/bufferline.h"
+#include "core/buffer_storage.h"
+#include "core/context.h"
+#include "core/devformat.h"
+#include "core/device.h"
+#include "core/effectslot.h"
+#include "core/filters/splitter.h"
+#include "core/fmt_traits.h"
+#include "core/mixer.h"
+#include "intrusive_ptr.h"
 #include "polyphase_resampler.h"
+#include "vector.h"
 
 
 namespace {
 
 /* Convolution reverb is implemented using a segmented overlap-add method. The
- * impulse response is broken up into multiple segments of 512 samples, and
- * each segment has an FFT applied with a 1024-sample buffer (the latter half
+ * impulse response is broken up into multiple segments of 128 samples, and
+ * each segment has an FFT applied with a 256-sample buffer (the latter half
  * left silent) to get its frequency-domain response. The resulting response
- * has its positive/non-mirrored frequencies saved (513 bins) in each segment.
+ * has its positive/non-mirrored frequencies saved (129 bins) in each segment.
  *
- * Input samples are similarly broken up into 512-sample segments, with an FFT
- * applied to each new incoming segment to get its 513 bins. A history of FFT'd
+ * Input samples are similarly broken up into 128-sample segments, with an FFT
+ * applied to each new incoming segment to get its 129 bins. A history of FFT'd
  * input segments is maintained, equal to the length of the impulse response.
  *
  * To apply the reverberation, each impulse response segment is convolved with
  * its paired input segment (using complex multiplies, far cheaper than FIRs),
- * accumulating into a 1024-bin FFT buffer. The input history is then shifted
- * to align with later impulse response segments for next time.
+ * accumulating into a 256-bin FFT buffer. The input history is then shifted to
+ * align with later impulse response segments for next time.
  *
- * An inverse FFT is then applied to the accumulated FFT buffer to get a 1024-
+ * An inverse FFT is then applied to the accumulated FFT buffer to get a 256-
  * sample time-domain response for output, which is split in two halves. The
- * first half is the 512-sample output, and the second half is a 512-sample
- * (really, 511) delayed extension, which gets added to the output next time.
+ * first half is the 128-sample output, and the second half is a 128-sample
+ * (really, 127) delayed extension, which gets added to the output next time.
  * Convolving two time-domain responses of lengths N and M results in a time-
  * domain signal of length N+M-1, and this holds true regardless of the
  * convolution being applied in the frequency domain, so these "overflow"
@@ -57,7 +72,7 @@ namespace {
  */
 
 
-void LoadSamples(double *RESTRICT dst, const al::byte *src, const size_t srcstep, FmtType srctype,
+void LoadSamples(float *RESTRICT dst, const al::byte *src, const size_t srcstep, FmtType srctype,
     const size_t samples) noexcept
 {
 #define HANDLE_FMT(T)  case T: al::LoadSampleArray<T>(dst, src, srcstep, samples); break
@@ -74,23 +89,28 @@ void LoadSamples(double *RESTRICT dst, const al::byte *src, const size_t srcstep
 }
 
 
-auto GetAmbiScales(AmbiScaling scaletype) noexcept -> const std::array<float,MAX_AMBI_CHANNELS>&
+inline auto& GetAmbiScales(AmbiScaling scaletype) noexcept
 {
-    if(scaletype == AmbiScaling::FuMa) return AmbiScale::FromFuMa;
-    if(scaletype == AmbiScaling::SN3D) return AmbiScale::FromSN3D;
-    return AmbiScale::FromN3D;
+    switch(scaletype)
+    {
+    case AmbiScaling::FuMa: return AmbiScale::FromFuMa();
+    case AmbiScaling::SN3D: return AmbiScale::FromSN3D();
+    case AmbiScaling::UHJ: return AmbiScale::FromUHJ();
+    case AmbiScaling::N3D: break;
+    }
+    return AmbiScale::FromN3D();
 }
 
-auto GetAmbiLayout(AmbiLayout layouttype) noexcept -> const std::array<uint8_t,MAX_AMBI_CHANNELS>&
+inline auto& GetAmbiLayout(AmbiLayout layouttype) noexcept
 {
-    if(layouttype == AmbiLayout::FuMa) return AmbiIndex::FromFuMa;
-    return AmbiIndex::FromACN;
+    if(layouttype == AmbiLayout::FuMa) return AmbiIndex::FromFuMa();
+    return AmbiIndex::FromACN();
 }
 
-auto GetAmbi2DLayout(AmbiLayout layouttype) noexcept -> const std::array<uint8_t,MAX_AMBI2D_CHANNELS>&
+inline auto& GetAmbi2DLayout(AmbiLayout layouttype) noexcept
 {
-    if(layouttype == AmbiLayout::FuMa) return AmbiIndex::FromFuMa2D;
-    return AmbiIndex::From2D;
+    if(layouttype == AmbiLayout::FuMa) return AmbiIndex::FromFuMa2D();
+    return AmbiIndex::FromACN2D();
 }
 
 
@@ -100,9 +120,13 @@ struct ChanMap {
     float elevation;
 };
 
-using complex_d = std::complex<double>;
+constexpr float Deg2Rad(float x) noexcept
+{ return static_cast<float>(al::numbers::pi / 180.0 * x); }
 
-constexpr size_t ConvolveUpdateSize{1024};
+
+using complex_f = std::complex<float>;
+
+constexpr size_t ConvolveUpdateSize{256};
 constexpr size_t ConvolveUpdateSamples{ConvolveUpdateSize / 2};
 
 
@@ -126,6 +150,19 @@ void apply_fir(al::span<float> dst, const float *RESTRICT src, const float *REST
         ++src;
     }
 
+#elif defined(HAVE_NEON)
+
+    for(float &output : dst)
+    {
+        float32x4_t r4{vdupq_n_f32(0.0f)};
+        for(size_t j{0};j < ConvolveUpdateSamples;j+=4)
+            r4 = vmlaq_f32(r4, vld1q_f32(&src[j]), vld1q_f32(&filter[j]));
+        r4 = vaddq_f32(r4, vrev64q_f32(r4));
+        output = vget_lane_f32(vadd_f32(vget_low_f32(r4), vget_high_f32(r4)), 0);
+
+        ++src;
+    }
+
 #else
 
     for(float &output : dst)
@@ -143,28 +180,28 @@ struct ConvolutionState final : public EffectState {
     FmtChannels mChannels{};
     AmbiLayout mAmbiLayout{};
     AmbiScaling mAmbiScaling{};
-    ALuint mAmbiOrder{};
+    uint mAmbiOrder{};
 
     size_t mFifoPos{0};
     std::array<float,ConvolveUpdateSamples*2> mInput{};
     al::vector<std::array<float,ConvolveUpdateSamples>,16> mFilter;
     al::vector<std::array<float,ConvolveUpdateSamples*2>,16> mOutput;
 
-    alignas(16) std::array<complex_d,ConvolveUpdateSize> mFftBuffer{};
+    alignas(16) std::array<complex_f,ConvolveUpdateSize> mFftBuffer{};
 
     size_t mCurrentSegment{0};
     size_t mNumConvolveSegs{0};
 
     struct ChannelData {
         alignas(16) FloatBufferLine mBuffer{};
-        float mHfScale{};
+        float mHfScale{}, mLfScale{};
         BandSplitter mFilter{};
         float Current[MAX_OUTPUT_CHANNELS]{};
         float Target[MAX_OUTPUT_CHANNELS]{};
     };
     using ChannelDataArray = al::FlexArray<ChannelData>;
     std::unique_ptr<ChannelDataArray> mChans;
-    std::unique_ptr<complex_d[]> mComplexData;
+    std::unique_ptr<complex_f[]> mComplexData;
 
 
     ConvolutionState() = default;
@@ -175,10 +212,11 @@ struct ConvolutionState final : public EffectState {
     void (ConvolutionState::*mMix)(const al::span<FloatBufferLine>,const size_t)
     {&ConvolutionState::NormalMix};
 
-    void deviceUpdate(const ALCdevice *device) override;
-    void setBuffer(const ALCdevice *device, const BufferStorage *buffer) override;
-    void update(const ALCcontext *context, const ALeffectslot *slot, const EffectProps *props, const EffectTarget target) override;
-    void process(const size_t samplesToDo, const al::span<const FloatBufferLine> samplesIn, const al::span<FloatBufferLine> samplesOut) override;
+    void deviceUpdate(const DeviceBase *device, const Buffer &buffer) override;
+    void update(const ContextBase *context, const EffectSlot *slot, const EffectProps *props,
+        const EffectTarget target) override;
+    void process(const size_t samplesToDo, const al::span<const FloatBufferLine> samplesIn,
+        const al::span<FloatBufferLine> samplesOut) override;
 
     DEF_NEWDEL(ConvolutionState)
 };
@@ -197,25 +235,24 @@ void ConvolutionState::UpsampleMix(const al::span<FloatBufferLine> samplesOut,
     for(auto &chan : *mChans)
     {
         const al::span<float> src{chan.mBuffer.data(), samplesToDo};
-        chan.mFilter.processHfScale(src, chan.mHfScale);
+        chan.mFilter.processScale(src, chan.mHfScale, chan.mLfScale);
         MixSamples(src, samplesOut, chan.Current, chan.Target, samplesToDo, 0);
     }
 }
 
 
-void ConvolutionState::deviceUpdate(const ALCdevice* /*device*/)
+void ConvolutionState::deviceUpdate(const DeviceBase *device, const Buffer &buffer)
 {
-}
+    using UhjDecoderType = UhjDecoder<512>;
+    static constexpr auto DecoderPadding = UhjDecoderType::sInputPadding;
 
-void ConvolutionState::setBuffer(const ALCdevice *device, const BufferStorage *buffer)
-{
-    constexpr ALuint MaxConvolveAmbiOrder{1u};
+    constexpr uint MaxConvolveAmbiOrder{1u};
 
     mFifoPos = 0;
     mInput.fill(0.0f);
     decltype(mFilter){}.swap(mFilter);
     decltype(mOutput){}.swap(mOutput);
-    mFftBuffer.fill(complex_d{});
+    mFftBuffer.fill(complex_f{});
 
     mCurrentSegment = 0;
     mNumConvolveSegs = 0;
@@ -224,29 +261,33 @@ void ConvolutionState::setBuffer(const ALCdevice *device, const BufferStorage *b
     mComplexData = nullptr;
 
     /* An empty buffer doesn't need a convolution filter. */
-    if(!buffer || buffer->mSampleLen < 1) return;
+    if(!buffer.storage || buffer.storage->mSampleLen < 1) return;
+
+    mChannels = buffer.storage->mChannels;
+    mAmbiLayout = IsUHJ(mChannels) ? AmbiLayout::FuMa : buffer.storage->mAmbiLayout;
+    mAmbiScaling = IsUHJ(mChannels) ? AmbiScaling::UHJ : buffer.storage->mAmbiScaling;
+    mAmbiOrder = minu(buffer.storage->mAmbiOrder, MaxConvolveAmbiOrder);
 
     constexpr size_t m{ConvolveUpdateSize/2 + 1};
-    auto bytesPerSample = BytesFromFmt(buffer->mType);
-    auto realChannels = ChannelsFromFmt(buffer->mChannels, buffer->mAmbiOrder);
-    auto numChannels = ChannelsFromFmt(buffer->mChannels,
-        minu(buffer->mAmbiOrder, MaxConvolveAmbiOrder));
+    const auto bytesPerSample = BytesFromFmt(buffer.storage->mType);
+    const auto realChannels = buffer.storage->channelsFromFmt();
+    const auto numChannels = (mChannels == FmtUHJ2) ? 3u : ChannelsFromFmt(mChannels, mAmbiOrder);
 
     mChans = ChannelDataArray::Create(numChannels);
 
     /* The impulse response needs to have the same sample rate as the input and
      * output. The bsinc24 resampler is decent, but there is high-frequency
-     * attenation that some people may be able to pick up on. Since this is
+     * attenuation that some people may be able to pick up on. Since this is
      * called very infrequently, go ahead and use the polyphase resampler.
      */
     PPhaseResampler resampler;
-    if(device->Frequency != buffer->mSampleRate)
-        resampler.init(buffer->mSampleRate, device->Frequency);
-    const auto resampledCount = static_cast<ALuint>(
-        (uint64_t{buffer->mSampleLen}*device->Frequency + (buffer->mSampleRate-1)) /
-        buffer->mSampleRate);
+    if(device->Frequency != buffer.storage->mSampleRate)
+        resampler.init(buffer.storage->mSampleRate, device->Frequency);
+    const auto resampledCount = static_cast<uint>(
+        (uint64_t{buffer.storage->mSampleLen}*device->Frequency+(buffer.storage->mSampleRate-1)) /
+        buffer.storage->mSampleRate);
 
-    const BandSplitter splitter{400.0f / static_cast<float>(device->Frequency)};
+    const BandSplitter splitter{device->mXOverFreq / static_cast<float>(device->Frequency)};
     for(auto &e : *mChans)
         e.mFilter = splitter;
 
@@ -262,49 +303,69 @@ void ConvolutionState::setBuffer(const ALCdevice *device, const BufferStorage *b
     mNumConvolveSegs = maxz(mNumConvolveSegs, 2) - 1;
 
     const size_t complex_length{mNumConvolveSegs * m * (numChannels+1)};
-    mComplexData = std::make_unique<complex_d[]>(complex_length);
-    std::fill_n(mComplexData.get(), complex_length, complex_d{});
+    mComplexData = std::make_unique<complex_f[]>(complex_length);
+    std::fill_n(mComplexData.get(), complex_length, complex_f{});
 
-    mChannels = buffer->mChannels;
-    mAmbiLayout = buffer->mAmbiLayout;
-    mAmbiScaling = buffer->mAmbiScaling;
-    mAmbiOrder = minu(buffer->mAmbiOrder, MaxConvolveAmbiOrder);
+    /* Load the samples from the buffer. */
+    const size_t srclinelength{RoundUp(buffer.storage->mSampleLen+DecoderPadding, 16)};
+    auto srcsamples = std::make_unique<float[]>(srclinelength * numChannels);
+    std::fill_n(srcsamples.get(), srclinelength * numChannels, 0.0f);
+    for(size_t c{0};c < numChannels && c < realChannels;++c)
+        LoadSamples(srcsamples.get() + srclinelength*c, buffer.samples.data() + bytesPerSample*c,
+            realChannels, buffer.storage->mType, buffer.storage->mSampleLen);
 
-    auto srcsamples = std::make_unique<double[]>(maxz(buffer->mSampleLen, resampledCount));
-    complex_d *filteriter = mComplexData.get() + mNumConvolveSegs*m;
+    if(IsUHJ(mChannels))
+    {
+        auto decoder = std::make_unique<UhjDecoderType>();
+        std::array<float*,4> samples{};
+        for(size_t c{0};c < numChannels;++c)
+            samples[c] = srcsamples.get() + srclinelength*c;
+        decoder->decode({samples.data(), numChannels}, buffer.storage->mSampleLen,
+            buffer.storage->mSampleLen);
+    }
+
+    auto ressamples = std::make_unique<double[]>(buffer.storage->mSampleLen +
+        (resampler ? resampledCount : 0));
+    complex_f *filteriter = mComplexData.get() + mNumConvolveSegs*m;
     for(size_t c{0};c < numChannels;++c)
     {
-        /* Load the samples from the buffer, and resample to match the device. */
-        LoadSamples(srcsamples.get(), buffer->mData.data() + bytesPerSample*c, realChannels,
-            buffer->mType, buffer->mSampleLen);
-        if(device->Frequency != buffer->mSampleRate)
-            resampler.process(buffer->mSampleLen, srcsamples.get(), resampledCount,
-                srcsamples.get());
+        /* Resample to match the device. */
+        if(resampler)
+        {
+            std::copy_n(srcsamples.get() + srclinelength*c, buffer.storage->mSampleLen,
+                ressamples.get() + resampledCount);
+            resampler.process(buffer.storage->mSampleLen, ressamples.get()+resampledCount,
+                resampledCount, ressamples.get());
+        }
+        else
+            std::copy_n(srcsamples.get() + srclinelength*c, buffer.storage->mSampleLen,
+                ressamples.get());
 
         /* Store the first segment's samples in reverse in the time-domain, to
          * apply as a FIR filter.
          */
         const size_t first_size{minz(resampledCount, ConvolveUpdateSamples)};
-        std::transform(srcsamples.get(), srcsamples.get()+first_size, mFilter[c].rbegin(),
+        std::transform(ressamples.get(), ressamples.get()+first_size, mFilter[c].rbegin(),
             [](const double d) noexcept -> float { return static_cast<float>(d); });
 
+        auto fftbuffer = std::vector<std::complex<double>>(ConvolveUpdateSize);
         size_t done{first_size};
         for(size_t s{0};s < mNumConvolveSegs;++s)
         {
             const size_t todo{minz(resampledCount-done, ConvolveUpdateSamples)};
 
-            auto iter = std::copy_n(&srcsamples[done], todo, mFftBuffer.begin());
+            auto iter = std::copy_n(&ressamples[done], todo, fftbuffer.begin());
             done += todo;
-            std::fill(iter, mFftBuffer.end(), complex_d{});
+            std::fill(iter, fftbuffer.end(), std::complex<double>{});
 
-            forward_fft(mFftBuffer);
-            filteriter = std::copy_n(mFftBuffer.cbegin(), m, filteriter);
+            forward_fft(al::as_span(fftbuffer));
+            filteriter = std::copy_n(fftbuffer.cbegin(), m, filteriter);
         }
     }
 }
 
 
-void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slot,
+void ConvolutionState::update(const ContextBase *context, const EffectSlot *slot,
     const EffectProps* /*props*/, const EffectTarget target)
 {
     /* NOTE: Stereo and Rear are slightly different from normal mixing (as
@@ -315,7 +376,7 @@ void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slo
      * to have its own output target since the main mixing buffer won't have an
      * LFE channel (due to being B-Format).
      */
-    static const ChanMap MonoMap[1]{
+    static constexpr ChanMap MonoMap[1]{
         { FrontCenter, 0.0f, 0.0f }
     }, StereoMap[2]{
         { FrontLeft,  Deg2Rad(-45.0f), Deg2Rad(0.0f) },
@@ -354,33 +415,48 @@ void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slo
         { SideRight,   Deg2Rad(  90.0f), Deg2Rad(0.0f) }
     };
 
-    if(mNumConvolveSegs < 1)
+    if(mNumConvolveSegs < 1) [[unlikely]]
         return;
 
     mMix = &ConvolutionState::NormalMix;
 
     for(auto &chan : *mChans)
         std::fill(std::begin(chan.Target), std::end(chan.Target), 0.0f);
-    const float gain{slot->Params.Gain};
-    if(mChannels == FmtBFormat3D || mChannels == FmtBFormat2D)
+    const float gain{slot->Gain};
+    if(IsAmbisonic(mChannels))
     {
-        ALCdevice *device{context->mDevice.get()};
-        if(device->mAmbiOrder > mAmbiOrder)
+        DeviceBase *device{context->mDevice};
+        if(mChannels == FmtUHJ2 && !device->mUhjEncoder)
         {
             mMix = &ConvolutionState::UpsampleMix;
-            const auto scales = BFormatDec::GetHFOrderScales(mAmbiOrder, device->mAmbiOrder);
+            (*mChans)[0].mHfScale = 1.0f;
+            (*mChans)[0].mLfScale = DecoderBase::sWLFScale;
+            (*mChans)[1].mHfScale = 1.0f;
+            (*mChans)[1].mLfScale = DecoderBase::sXYLFScale;
+            (*mChans)[2].mHfScale = 1.0f;
+            (*mChans)[2].mLfScale = DecoderBase::sXYLFScale;
+        }
+        else if(device->mAmbiOrder > mAmbiOrder)
+        {
+            mMix = &ConvolutionState::UpsampleMix;
+            const auto scales = AmbiScale::GetHFOrderScales(mAmbiOrder, device->mAmbiOrder,
+                device->m2DMixing);
             (*mChans)[0].mHfScale = scales[0];
+            (*mChans)[0].mLfScale = 1.0f;
             for(size_t i{1};i < mChans->size();++i)
+            {
                 (*mChans)[i].mHfScale = scales[1];
+                (*mChans)[i].mLfScale = 1.0f;
+            }
         }
         mOutTarget = target.Main->Buffer;
 
-        const auto &scales = GetAmbiScales(mAmbiScaling);
-        const uint8_t *index_map{(mChannels == FmtBFormat2D) ?
+        auto&& scales = GetAmbiScales(mAmbiScaling);
+        const uint8_t *index_map{Is2DAmbisonic(mChannels) ?
             GetAmbi2DLayout(mAmbiLayout).data() :
             GetAmbiLayout(mAmbiLayout).data()};
 
-        std::array<float,MAX_AMBI_CHANNELS> coeffs{};
+        std::array<float,MaxAmbiChannels> coeffs{};
         for(size_t c{0u};c < mChans->size();++c)
         {
             const size_t acn{index_map[c]};
@@ -391,11 +467,12 @@ void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slo
     }
     else
     {
-        ALCdevice *device{context->mDevice.get()};
+        DeviceBase *device{context->mDevice};
         al::span<const ChanMap> chanmap{};
         switch(mChannels)
         {
         case FmtMono: chanmap = MonoMap; break;
+        case FmtSuperStereo:
         case FmtStereo: chanmap = StereoMap; break;
         case FmtRear: chanmap = RearMap; break;
         case FmtQuad: chanmap = QuadMap; break;
@@ -404,6 +481,9 @@ void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slo
         case FmtX71: chanmap = X71Map; break;
         case FmtBFormat2D:
         case FmtBFormat3D:
+        case FmtUHJ2:
+        case FmtUHJ3:
+        case FmtUHJ4:
             break;
         }
 
@@ -412,9 +492,10 @@ void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slo
         {
             auto ScaleAzimuthFront = [](float azimuth, float scale) -> float
             {
+                constexpr float half_pi{al::numbers::pi_v<float>*0.5f};
                 const float abs_azi{std::fabs(azimuth)};
-                if(!(abs_azi >= al::MathDefs<float>::Pi()*0.5f))
-                    return std::copysign(minf(abs_azi*scale, al::MathDefs<float>::Pi()*0.5f), azimuth);
+                if(!(abs_azi >= half_pi))
+                    return std::copysign(minf(abs_azi*scale, half_pi), azimuth);
                 return azimuth;
             };
 
@@ -438,7 +519,7 @@ void ConvolutionState::update(const ALCcontext *context, const ALeffectslot *slo
 void ConvolutionState::process(const size_t samplesToDo,
     const al::span<const FloatBufferLine> samplesIn, const al::span<FloatBufferLine> samplesOut)
 {
-    if(mNumConvolveSegs < 1)
+    if(mNumConvolveSegs < 1) [[unlikely]]
         return;
 
     constexpr size_t m{ConvolveUpdateSize/2 + 1};
@@ -458,8 +539,7 @@ void ConvolutionState::process(const size_t samplesToDo,
         for(size_t c{0};c < chans.size();++c)
         {
             auto buf_iter = chans[c].mBuffer.begin() + base;
-            apply_fir({std::addressof(*buf_iter), todo}, mInput.data()+1 + mFifoPos,
-                mFilter[c].data());
+            apply_fir({buf_iter, todo}, mInput.data()+1 + mFifoPos, mFilter[c].data());
 
             auto fifo_iter = mOutput[c].begin() + mFifoPos;
             std::transform(fifo_iter, fifo_iter+todo, buf_iter, buf_iter, std::plus<>{});
@@ -479,20 +559,20 @@ void ConvolutionState::process(const size_t samplesToDo,
          * frequency bins to the FFT history.
          */
         auto fftiter = std::copy_n(mInput.cbegin(), ConvolveUpdateSamples, mFftBuffer.begin());
-        std::fill(fftiter, mFftBuffer.end(), complex_d{});
-        forward_fft(mFftBuffer);
+        std::fill(fftiter, mFftBuffer.end(), complex_f{});
+        forward_fft(al::as_span(mFftBuffer));
 
         std::copy_n(mFftBuffer.cbegin(), m, &mComplexData[curseg*m]);
 
-        const complex_d *RESTRICT filter{mComplexData.get() + mNumConvolveSegs*m};
+        const complex_f *RESTRICT filter{mComplexData.get() + mNumConvolveSegs*m};
         for(size_t c{0};c < chans.size();++c)
         {
-            std::fill_n(mFftBuffer.begin(), m, complex_d{});
+            std::fill_n(mFftBuffer.begin(), m, complex_f{});
 
             /* Convolve each input segment with its IR filter counterpart
              * (aligned in time).
              */
-            const complex_d *RESTRICT input{&mComplexData[curseg*m]};
+            const complex_f *RESTRICT input{&mComplexData[curseg*m]};
             for(size_t s{curseg};s < mNumConvolveSegs;++s)
             {
                 for(size_t i{0};i < m;++i,++input,++filter)
@@ -511,24 +591,22 @@ void ConvolutionState::process(const size_t samplesToDo,
             for(size_t i{m};i < ConvolveUpdateSize;++i)
                 mFftBuffer[i] = std::conj(mFftBuffer[ConvolveUpdateSize-i]);
 
-            /* Apply iFFT to get the 1024 (really 1023) samples for output. The
-             * 512 output samples are combined with the last output's 511
+            /* Apply iFFT to get the 256 (really 255) samples for output. The
+             * 128 output samples are combined with the last output's 127
              * second-half samples (and this output's second half is
              * subsequently saved for next time).
              */
-            inverse_fft(mFftBuffer);
+            inverse_fft(al::as_span(mFftBuffer));
 
             /* The iFFT'd response is scaled up by the number of bins, so apply
              * the inverse to normalize the output.
              */
             for(size_t i{0};i < ConvolveUpdateSamples;++i)
                 mOutput[c][i] =
-                    static_cast<float>(mFftBuffer[i].real() * (1.0/double{ConvolveUpdateSize})) +
-                    mOutput[c][ConvolveUpdateSamples+i];
+                    (mFftBuffer[i].real()+mOutput[c][ConvolveUpdateSamples+i]) *
+                    (1.0f/float{ConvolveUpdateSize});
             for(size_t i{0};i < ConvolveUpdateSamples;++i)
-                mOutput[c][ConvolveUpdateSamples+i] =
-                    static_cast<float>(mFftBuffer[ConvolveUpdateSamples+i].real() *
-                        (1.0/double{ConvolveUpdateSize}));
+                mOutput[c][ConvolveUpdateSamples+i] = mFftBuffer[ConvolveUpdateSamples+i].real();
         }
 
         /* Shift the input history. */
@@ -541,102 +619,11 @@ void ConvolutionState::process(const size_t samplesToDo,
 }
 
 
-void ConvolutionEffect_setParami(EffectProps* /*props*/, ALenum param, int /*val*/)
-{
-    switch(param)
-    {
-    default:
-        throw effect_exception{AL_INVALID_ENUM, "Invalid null effect integer property 0x%04x",
-            param};
-    }
-}
-void ConvolutionEffect_setParamiv(EffectProps *props, ALenum param, const int *vals)
-{
-    switch(param)
-    {
-    default:
-        ConvolutionEffect_setParami(props, param, vals[0]);
-    }
-}
-void ConvolutionEffect_setParamf(EffectProps* /*props*/, ALenum param, float /*val*/)
-{
-    switch(param)
-    {
-    default:
-        throw effect_exception{AL_INVALID_ENUM, "Invalid null effect float property 0x%04x",
-            param};
-    }
-}
-void ConvolutionEffect_setParamfv(EffectProps *props, ALenum param, const float *vals)
-{
-    switch(param)
-    {
-    default:
-        ConvolutionEffect_setParamf(props, param, vals[0]);
-    }
-}
-
-void ConvolutionEffect_getParami(const EffectProps* /*props*/, ALenum param, int* /*val*/)
-{
-    switch(param)
-    {
-    default:
-        throw effect_exception{AL_INVALID_ENUM, "Invalid null effect integer property 0x%04x",
-            param};
-    }
-}
-void ConvolutionEffect_getParamiv(const EffectProps *props, ALenum param, int *vals)
-{
-    switch(param)
-    {
-    default:
-        ConvolutionEffect_getParami(props, param, vals);
-    }
-}
-void ConvolutionEffect_getParamf(const EffectProps* /*props*/, ALenum param, float* /*val*/)
-{
-    switch(param)
-    {
-    default:
-        throw effect_exception{AL_INVALID_ENUM, "Invalid null effect float property 0x%04x",
-            param};
-    }
-}
-void ConvolutionEffect_getParamfv(const EffectProps *props, ALenum param, float *vals)
-{
-    switch(param)
-    {
-    default:
-        ConvolutionEffect_getParamf(props, param, vals);
-    }
-}
-
-DEFINE_ALEFFECT_VTABLE(ConvolutionEffect);
-
-
 struct ConvolutionStateFactory final : public EffectStateFactory {
-    EffectState *create() override;
-    EffectProps getDefaultProps() const noexcept override;
-    const EffectVtable *getEffectVtable() const noexcept override;
+    al::intrusive_ptr<EffectState> create() override
+    { return al::intrusive_ptr<EffectState>{new ConvolutionState{}}; }
 };
 
-/* Creates EffectState objects of the appropriate type. */
-EffectState *ConvolutionStateFactory::create()
-{ return new ConvolutionState{}; }
-
-/* Returns an ALeffectProps initialized with this effect type's default
- * property values.
- */
-EffectProps ConvolutionStateFactory::getDefaultProps() const noexcept
-{
-    EffectProps props{};
-    return props;
-}
-
-/* Returns a pointer to this effect type's global set/get vtable. */
-const EffectVtable *ConvolutionStateFactory::getEffectVtable() const noexcept
-{ return &ConvolutionEffect_vtable; }
-
 } // namespace
 
 EffectStateFactory *ConvolutionStateFactory_getFactory()
